/*
 *    Copyright (c) 2003 by Axys Design Automation, Inc.,
 *    Irvine, Ca, USA and
 *    AXYS GmbH, Herzogenrath, Germany
 *
 *    AXYS Design Automation Inc. / AXYS GmbH Proprietary Information.
 *    Disclosure to third parties is prohibited without prior consent.
 *
 *    Example code for ARM CLCD Primecell
 *    
 */

/*#include "AXYS-Logo.h"*/
#include "msoc_video_in.h"
#include "msoc_video_inG.h"
#include "stdio.h"

#define FB_BASE                     0xC2000000
#define FB_BASE2                    0x82000000
#define CLCDC_BASE                  0x10102000
#define AHB_WRITE(address, value)   (*((unsigned int*)(address))) = (value)

#define TIMER_OFFSET                0x20000000
#define rTIMER_ENA                  (*((volatile unsigned int *)(TIMER_OFFSET)))
#define rTIMER_THRESHOLD            (*((volatile unsigned int *)(TIMER_OFFSET+0x4)))
#define rTIMER_STATUS               (*((volatile unsigned int *)(TIMER_OFFSET+0x8)))
#define rTIMER_INTCLR               (*((volatile unsigned int *)(TIMER_OFFSET+0xc)))

typedef unsigned int    U32;
static unsigned int     hour;
static unsigned int     minute;
static unsigned int     frame_no_fg;

/*******************************************************
    ISR
********************************************************/
__irq void IRQ_Handler(void)
{
    rTIMER_INTCLR = 1;
    
    if(minute == 59)
    {
        minute = 0;
        
        if(hour == 24)
            hour = 0;
        else
            hour++;
    }
    else
        minute++;
        
    /* dummy code */  
    frame_no_fg++;
    if(frame_no_fg == FRAME_AMOUNTG)
        frame_no_fg = 0;        
    /* dummy code */
        
    while(1)
    {
        if(rTIMER_STATUS == 0)
            break;
    }
    
    rTIMER_INTCLR = 0;
    
    return;
}

vframe * construct_timer_frame()
{
    // Please generate time display for example "10:10" 
    // in size 320x200 according to variables "hour" and "minute" here
    
    // dummy code
    return (vframe *)array_pframeG[frame_no_fg];    
}

void
InitFrameBuffer(const vframe* pframe, const vframe* pframe_fg, 
                unsigned int alpha, unsigned int base)
{
    U32 i, n, nrBytes;
    
    nrBytes = pframe->height * pframe->width * pframe->bytes_per_pixel;

    
    printf("initialize frame begin......\n");
    // transform 24bpp to 16bpp
    for (i = 0, n = 0; i < nrBytes; i+=pframe->bytes_per_pixel*2, n += 4)
    {
        U32 r0, g0, b0, r1, g1, b1, wordOut;
         
        // alpha blending with alpha = 0.5
        r0 = (U32)((pframe->pixel_data[i+0]>>1) + (pframe_fg->pixel_data[i+0]>>1));
        g0 = (U32)((pframe->pixel_data[i+1]>>1) + (pframe_fg->pixel_data[i+1]>>1));
        b0 = (U32)((pframe->pixel_data[i+2]>>1) + (pframe_fg->pixel_data[i+2]>>1));

        r1 = (U32)((pframe->pixel_data[i+3]>>1) + (pframe_fg->pixel_data[i+3]>>1));
        g1 = (U32)((pframe->pixel_data[i+4]>>1) + (pframe_fg->pixel_data[i+4]>>1));
        b1 = (U32)((pframe->pixel_data[i+5]>>1) + (pframe_fg->pixel_data[i+5]>>1));

/*
        r0 = (U32)pframe->pixel_data[i+0];
        g0 = (U32)pframe->pixel_data[i+1];
        b0 = (U32)pframe->pixel_data[i+2];

        r1 = (U32)pframe->pixel_data[i+3];
        g1 = (U32)pframe->pixel_data[i+4];
        b1 = (U32)pframe->pixel_data[i+5];
*/
        // 16 format is RGB 5:5:5
        wordOut = ((((b1 >> 3) << 10) | ((g1 >> 3) << 5) | (r1 >> 3)) << 16)
                 |((((b0 >> 3) << 10) | ((g0 >> 3) << 5) | (r0 >> 3)) <<  0);

        AHB_WRITE(base + n, wordOut);
    }
    
    printf("initialize frame OK!!!!!!\n");
}

void
InitCLCD(unsigned int base)
{
    // set up for TFT 320x200 16BPP no-palette
    U32 HBP = 0x8c;
    U32 HFP = 0x0;
    U32 HSW = 0x8c0;
    U32 PPL = 320 / 16 - 1;
    U32 LCDTiming0 = (HBP << 24) | (HFP << 16) | (HSW << 8) | (PPL << 2);

    U32 VBP = 0;
    U32 VFP = 0;
    U32 VSW = 200 - 1;
    U32 LPP = 200 - 1;
    U32 LCDTiming1 = (VBP << 24) | (VFP << 16) | (VSW << 10) | (LPP << 0);

    U32 BCD = 0;
    U32 CPL = 320;
    U32 IOE = 0;
    U32 IPC = 0;
    U32 IHS = 0;
    U32 IVS = 0;
    U32 ACB = 0;
    U32 CLKSEL = 0;
    U32 PCD = 16;
    U32 LCDTiming2 = (BCD << 26) | (CPL << 16) | (IOE << 14) | (IPC << 13) | (IHS << 12) | (IVS << 11) | (ACB << 6) | (CLKSEL << 5) | PCD;

    // see CLCD TRM
    AHB_WRITE(CLCDC_BASE + 0x00, LCDTiming0);
    AHB_WRITE(CLCDC_BASE + 0x04, LCDTiming1);
    AHB_WRITE(CLCDC_BASE + 0x08, LCDTiming2);
    AHB_WRITE(CLCDC_BASE + 0x0C, 0x00000000);
    AHB_WRITE(CLCDC_BASE + 0x10, base   ); // Framebuffer base
    AHB_WRITE(CLCDC_BASE + 0x14, 0x00000000);
    AHB_WRITE(CLCDC_BASE + 0x18, 0x00000000);
    AHB_WRITE(CLCDC_BASE + 0x1C, 0x00000869); // 16BPP TTF
}

int
main(int argc, char *argv[])
{
    unsigned int up_addr;
    int frame_no;
    vframe *out_frame_fg;
    
    if (argc); // suppress compiler warning
    if (argv); // suppress compiler warning

    hour = 0;
    minute = 0;
    frame_no_fg = 0;    
    
    rTIMER_THRESHOLD = 150000000;  // do not change this value
    rTIMER_ENA = 1; 
    
    frame_no = 0;
    init_frames();
    init_framesG();
    
    while(1)
    {
        out_frame_fg = construct_timer_frame();
        InitFrameBuffer(array_pframe[frame_no++], out_frame_fg, 8, FB_BASE);
        InitCLCD(FB_BASE);
        
        if(frame_no == FRAME_AMOUNT)
            frame_no = 0;
        
        
        while(1)
        {
            up_addr = (*((volatile unsigned int*)(CLCDC_BASE + 0x28)));
            
            if(up_addr >= ((FB_BASE+( 320*200*2 ))-1) )
            break;
        }
        
        out_frame_fg = construct_timer_frame();
        InitFrameBuffer(array_pframe[frame_no++], out_frame_fg, 8, FB_BASE2);
        InitCLCD(FB_BASE2);
        
        if(frame_no == FRAME_AMOUNT)
            frame_no = 0;       
        
        while(1)
        {
            up_addr = (*((volatile unsigned int*)(CLCDC_BASE + 0x28)));
            
            if(up_addr >= ((FB_BASE2+( 320*200*2 ))-1) )
            break;
        }       
    }
    
    return 0;
}
